@@ -1,25 +1,38 @@
-use crossterm::event::{self, Event as CrosstermEvent};
-use frep_core::validation::SearchConfiguration;
-use futures::Stream;
-use futures::StreamExt;
-use log::error;
-use log::LevelFilter;
-use ratatui::backend::Backend;
-use ratatui::backend::TestBackend;
-use ratatui::crossterm::event::KeyEventKind;
-use ratatui::{backend::CrosstermBackend, Terminal};
-use std::env;
-use std::io;
-use std::path::Path;
-use std::path::PathBuf;
-use std::process::Command;
-use std::str::FromStr;
-use tokio::sync::mpsc::UnboundedReceiver;
-use tokio::sync::mpsc::UnboundedSender;
+use anyhow::Context;
+use crossterm::{
+    event::{self, Event as CrosstermEvent},
+    style::Stylize as _,
+};
+use frep_core::{replace::ReplaceResult, search::SearchResultWithReplacement};
+use futures::{Stream, StreamExt};
+use log::{error, LevelFilter};
+use ratatui::{
+    backend::{Backend, CrosstermBackend, TestBackend},
+    crossterm::event::KeyEventKind,
+    Terminal,
+};
+use scooter_core::{
+    app::{
+        App, AppRunConfig, Event, EventHandlingResult, ExitAndReplaceState, ExitState, InputSource,
+    },
+    errors::AppError,
+    fields::SearchFieldValues,
+    replace::ReplaceState,
+};
+use std::{
+    collections::HashMap,
+    env,
+    io::{self, Write},
+    path::{Path, PathBuf},
+    process::Command,
+    str::FromStr,
+    sync::Arc,
+};
+use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender};
 
 use crate::{
-    app::{App, AppError, AppRunConfig, Event, EventHandlingResult},
-    fields::SearchFieldValues,
+    config::{load_config, Config},
+    conversions,
     logging::DEFAULT_LOG_LEVEL,
     tui::Tui,
 };
@@ -31,6 +44,7 @@ pub struct AppConfig<'a> {
     pub log_level: LevelFilter,
     pub search_field_values: SearchFieldValues<'a>,
     pub app_run_config: AppRunConfig,
+    pub stdin_content: Option<String>,
 }
 
 impl Default for AppConfig<'_> {
@@ -40,29 +54,11 @@ impl Default for AppConfig<'_> {
             log_level: LevelFilter::from_str(DEFAULT_LOG_LEVEL).unwrap(),
             search_field_values: SearchFieldValues::default(),
             app_run_config: AppRunConfig::default(),
+            stdin_content: None,
         }
     }
 }
 
-impl<'a> TryFrom<AppConfig<'a>> for SearchConfiguration<'a> {
-    type Error = anyhow::Error;
-
-    fn try_from(config: AppConfig<'a>) -> anyhow::Result<Self> {
-        Ok(SearchConfiguration {
-            search_text: config.search_field_values.search.value,
-            replacement_text: config.search_field_values.replace.value,
-            fixed_strings: config.search_field_values.fixed_strings.value,
-            advanced_regex: config.app_run_config.advanced_regex,
-            include_globs: Some(config.search_field_values.include_files.value),
-            exclude_globs: Some(config.search_field_values.exclude_files.value),
-            match_whole_word: config.search_field_values.match_whole_word.value,
-            match_case: config.search_field_values.match_case.value,
-            include_hidden: config.app_run_config.include_hidden,
-            directory: config.directory,
-        })
-    }
-}
-
 pub trait EventStream:
     Stream<Item = Result<CrosstermEvent, std::io::Error>> + Send + Unpin
 {
@@ -73,6 +69,7 @@ pub type CrosstermEventStream = event::EventStream;
 
 pub struct AppRunner<B: Backend, E: EventStream, S: SnapshotProvider<B>> {
     app: App,
+    config: Config,
     event_receiver: UnboundedReceiver<Event>,
     tui: Tui<B>,
     event_stream: E,
@@ -125,11 +122,18 @@ impl SnapshotProvider<TestBackend> for TestSnapshotProvider {
 }
 
 impl AppRunner<CrosstermBackend<io::Stdout>, CrosstermEventStream, NoOpSnapshotProvider> {
-    pub fn new_runner(config: AppConfig<'_>) -> anyhow::Result<Self> {
+    pub fn new_runner(app_config: AppConfig<'_>) -> anyhow::Result<Self> {
         let backend = CrosstermBackend::new(io::stdout());
         let event_stream = CrosstermEventStream::new();
         let snapshot_provider = NoOpSnapshotProvider;
-        Self::new(config, backend, event_stream, snapshot_provider)
+        let user_config = load_config().context("Failed to read config file")?;
+        Self::new(
+            app_config,
+            user_config,
+            backend,
+            event_stream,
+            snapshot_provider,
+        )
     }
 }
 
@@ -137,27 +141,43 @@ impl<E: EventStream> AppRunner<TestBackend, E, TestSnapshotProvider> {
     // Used in integration tests
     #[allow(dead_code)]
     pub fn new_test_with_snapshot(
-        config: AppConfig<'_>,
+        app_config: AppConfig<'_>,
         backend: TestBackend,
         event_stream: E,
         snapshot_sender: UnboundedSender<String>,
     ) -> anyhow::Result<Self> {
         let snapshot_provider = TestSnapshotProvider::new(snapshot_sender);
-        Self::new(config, backend, event_stream, snapshot_provider)
+        // Tests should use default config, not load from user's config directory
+        let test_config = Config::default();
+        Self::new(
+            app_config,
+            test_config,
+            backend,
+            event_stream,
+            snapshot_provider,
+        )
     }
 }
 
 impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B, E, S> {
     pub fn new(
-        config: AppConfig<'_>,
+        app_config: AppConfig<'_>,
+        config: Config,
         backend: B,
         event_stream: E,
         snapshot_provider: S,
     ) -> anyhow::Result<Self> {
+        let input_source = if let Some(stdin_content) = app_config.stdin_content {
+            InputSource::Stdin(Arc::new(stdin_content))
+        } else {
+            InputSource::Directory(app_config.directory.clone())
+        };
+
         let (app, event_receiver) = App::new_with_receiver(
-            config.directory,
-            &config.search_field_values,
-            &config.app_run_config,
+            input_source,
+            &app_config.search_field_values,
+            &app_config.app_run_config,
+            config.search.disable_prepopulated_fields,
         );
 
         let terminal = Terminal::new(backend)?;
@@ -165,6 +185,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
 
         Ok(Self {
             app,
+            config,
             event_receiver,
             tui,
             event_stream,
@@ -180,18 +201,22 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
     }
 
     pub fn draw(&mut self) -> anyhow::Result<()> {
-        self.tui.draw(&mut self.app)?;
+        self.tui.draw(&mut self.app, &self.config)?;
         self.snapshot_provider.send_snapshot(&self.tui);
         Ok(())
     }
 
-    pub async fn run_event_loop(&mut self) -> anyhow::Result<Option<String>> {
+    pub async fn run_event_loop(&mut self) -> anyhow::Result<Option<ExitState>> {
         loop {
             let event_handling_result = tokio::select! {
                 Some(Ok(event)) = self.event_stream.next() => {
                     match event {
                         CrosstermEvent::Key(key) if key.kind == KeyEventKind::Press => {
-                            self.app.handle_key_event(&key)
+                            if let Some((code, modifiers)) = conversions::convert_key_event(&key) {
+                                self.app.handle_key_event(code, modifiers)
+                            } else {
+                                EventHandlingResult::None
+                            }
                         },
                         CrosstermEvent::Resize(_, _) => EventHandlingResult::Rerender,
                         _ => EventHandlingResult::None,
@@ -204,7 +229,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
                             self.tui.show_cursor()?;
                             match self.open_editor(file_path, line) {
                                 Ok(()) => {
-                                    if self.app.config.editor_open.exit {
+                                    if self.config.editor_open.exit {
                                         res = EventHandlingResult::Exit(None);
                                     }
                                 }
@@ -220,33 +245,17 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
                             }
                             self.tui.init()?;
                             res
-
                         }
                         Event::App(app_event) => {
                             self.app.handle_app_event(&app_event)
                         }
                         Event::PerformReplacement => {
-                            if !self.app.is_search_complete() {
-                                self.app.add_error(AppError {
-                                    name: "Search still in progress".to_string(),
-                                    long: "Try again when search is complete".to_string(),
-                                });
-                            } else if !self.app.is_preview_updated() {
-                                self.app.add_error(AppError {
-                                    name: "Updating replacement preview".to_string(),
-                                    long: "Try again when complete".to_string(),
-                                });
-                            } else if !self.app.background_processing_reciever().is_some_and(|r| r.is_empty()) {
-                                self.app.add_error(AppError {
-                                    name: "Background processing in progress".to_string(),
-                                    long: "Try again in a moment".to_string(),
-                                });
-                            } else {
-                                self.app.perform_replacement();
-                            }
-
+                            self.app.perform_replacement();
                             EventHandlingResult::Rerender
                         }
+                        Event::ExitAndReplace(state) => {
+                            return Ok(Some(ExitState::StdinState(state)));
+                        }
                     }
                 }
                 Some(event) = self.app.background_processing_recv() => {
@@ -259,7 +268,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
 
             match event_handling_result {
                 EventHandlingResult::Rerender => self.draw()?,
-                EventHandlingResult::Exit(results) => return Ok(results),
+                EventHandlingResult::Exit(results) => return Ok(results.map(|t| *t)),
                 EventHandlingResult::None => {}
             }
         }
@@ -271,7 +280,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
     }
 
     fn open_editor(&self, file_path: PathBuf, line: usize) -> anyhow::Result<()> {
-        match &self.app.config.editor_open.command {
+        match &self.config.editor_open.command {
             Some(command) => {
                 Self::open_editor_from_command(command, &file_path, line)?;
             }
@@ -371,10 +380,172 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
     }
 }
 
+pub fn format_replacement_results(
+    num_successes: usize,
+    num_ignored: Option<usize>,
+    errors: Option<&[SearchResultWithReplacement]>,
+) -> String {
+    let errors_display = if let Some(errors) = errors {
+        #[allow(clippy::format_collect)]
+        errors
+            .iter()
+            .map(|error| {
+                let (path, error) = error.display_error();
+                format!("\n{path}:\n  {}", error.red())
+            })
+            .collect::<String>()
+    } else {
+        String::new()
+    };
+
+    let maybe_ignored_str = match num_ignored {
+        Some(n) => format!("\nIgnored (lines): {n}"),
+        None => "".into(),
+    };
+    let maybe_errors_str = match errors {
+        Some(errors) => format!(
+            "\nErrors: {num_errors}{errors_display}",
+            num_errors = errors.len()
+        ),
+        None => "".into(),
+    };
+
+    format!(
+        "Successful replacements (lines): {num_successes}{maybe_ignored_str}{maybe_errors_str}\n"
+    )
+}
+
 pub async fn run_app_tui(app_config: AppConfig<'_>) -> anyhow::Result<Option<String>> {
     let mut runner = AppRunner::new_runner(app_config)?;
     runner.init()?;
-    let results = runner.run_event_loop().await?;
+    let mut exit_state = runner.run_event_loop().await?;
     runner.cleanup()?;
-    Ok(results)
+
+    let stats = match exit_state {
+        Some(ExitState::Stats(stats)) => Some(stats),
+        Some(ExitState::StdinState(ref mut state)) => {
+            if runner.app.print_results {
+                let res = write_results_to_stderr_with_stats(state)?;
+                Some(res)
+            } else {
+                write_results_to_stderr(state)?;
+                None
+            }
+        }
+        None => None,
+    }
+    .map(|stats| {
+        format_replacement_results(
+            stats.num_successes,
+            Some(stats.num_ignored),
+            Some(&stats.errors),
+        )
+    });
+    Ok(stats)
+}
+
+fn write_results_to_stderr(state: &mut ExitAndReplaceState) -> anyhow::Result<()> {
+    write_results_to_stderr_impl(state, false).map(|res| {
+        assert!(res.is_none(), "Found Some stats, expected None");
+    })
+}
+
+fn write_results_to_stderr_with_stats(
+    state: &mut ExitAndReplaceState,
+) -> anyhow::Result<ReplaceState> {
+    write_results_to_stderr_impl(state, true)
+        .map(|res| res.expect("Found None stats, expected Some"))
+}
+
+fn write_results_to_stderr_impl(
+    state: &mut ExitAndReplaceState,
+    return_stats: bool,
+) -> anyhow::Result<Option<ReplaceState>> {
+    let mut num_successes = 0;
+    let mut num_ignored = 0;
+
+    let mut line_map = state
+        .replace_results
+        .iter_mut()
+        .map(|res| (res.search_result.line_number, res))
+        .collect::<HashMap<_, _>>();
+
+    for (idx, line) in state.stdin.lines().enumerate() {
+        let line_number = idx + 1; // Ensure line-number is 1-indexed
+        let line_new = line_map
+            .get_mut(&line_number)
+            .and_then(|res| {
+                assert_eq!(
+                    line, res.search_result.line,
+                    "line has changed since search"
+                );
+                if res.search_result.included {
+                    res.replace_result = Some(ReplaceResult::Success);
+                    num_successes += 1;
+                    Some(res.replacement.as_str())
+                } else {
+                    num_ignored += 1;
+                    None
+                }
+            })
+            .unwrap_or(line);
+        writeln!(io::stderr(), "{line_new}")?;
+    }
+
+    let res = if return_stats {
+        Some(ReplaceState {
+            num_successes,
+            num_ignored,
+            errors: Vec::new(),
+            replacement_errors_pos: 0,
+        })
+    } else {
+        None
+    };
+    Ok(res)
+}
+
+#[cfg(test)]
+mod tests {
+    use frep_core::{line_reader::LineEnding, replace::ReplaceResult, search::SearchResult};
+
+    use super::*;
+
+    #[test]
+    fn test_format_replacement_results_no_errors() {
+        let result = format_replacement_results(5, Some(2), Some(&[]));
+        assert_eq!(
+            result,
+            "Successful replacements (lines): 5\nIgnored (lines): 2\nErrors: 0\n"
+        );
+    }
+
+    #[test]
+    fn test_format_replacement_results_with_errors() {
+        let error_result = SearchResultWithReplacement {
+            search_result: SearchResult {
+                path: Some(PathBuf::from("file.txt")),
+                line_number: 10,
+                line: "line".to_string(),
+                line_ending: LineEnding::Lf,
+                included: true,
+            },
+            replacement: "replacement".to_string(),
+            replace_result: Some(ReplaceResult::Error("Test error".to_string())),
+        };
+
+        let result = format_replacement_results(3, Some(1), Some(&[error_result]));
+        assert!(result.contains("Successful replacements (lines): 3\n"));
+        assert!(result.contains("Ignored (lines): 1"));
+        assert!(result.contains("Errors: 1"));
+        assert!(result.contains("file.txt:10"));
+        assert!(result.contains("Test error"));
+    }
+
+    #[test]
+    fn test_format_replacement_results_no_ignored_count() {
+        let result = format_replacement_results(7, None, Some(&[]));
+        assert_eq!(result, "Successful replacements (lines): 7\nErrors: 0\n");
+        assert!(!result.contains("Ignored (lines):"));
+    }
 }
