@@ -14,11 +14,10 @@ use tokio::{
     time::{sleep, Duration, Instant},
 };
 
-use scooter::{
+use scooter::app_runner::{AppConfig, AppRunner};
+use scooter_core::{
     app::AppRunConfig,
-    app_runner::{AppConfig, AppRunner},
     fields::{FieldValue, SearchFieldValues},
-    test_with_both_regex_modes, test_with_both_regex_modes_and_fixed_strings,
 };
 
 mod utils;
@@ -265,7 +264,7 @@ async fn test_search_current_dir() -> anyhow::Result<()> {
     send_chars("search", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
 
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
@@ -322,7 +321,7 @@ test_with_both_regex_modes!(
         send_chars("after", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
 
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
@@ -391,7 +390,7 @@ test_with_both_regex_modes!(
         send_chars("after", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
 
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
@@ -438,7 +437,7 @@ test_with_both_regex_modes_and_fixed_strings!(
         }
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
 
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
@@ -489,7 +488,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
 
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
@@ -557,7 +556,7 @@ test_with_both_regex_modes!(
         send_chars("+44 $2 $1-$3", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         // Nothing should have changed yet
@@ -708,7 +707,7 @@ async fn test_multi_select_mode() -> anyhow::Result<()> {
     send_chars("changed", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     // Highlight 3rd to 6th search result with multi-select, and 8th with single selection
@@ -783,7 +782,7 @@ async fn test_results_calculation_mixed() -> anyhow::Result<()> {
     send_chars("changed", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     send_key(KeyCode::Char('j'), &event_sender);
@@ -849,7 +848,7 @@ async fn test_results_calculation_all_success() -> anyhow::Result<()> {
     send_chars("changed", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     send_key(KeyCode::Enter, &event_sender);
@@ -911,7 +910,7 @@ async fn test_results_calculation_all_ignored() -> anyhow::Result<()> {
     send_chars("changed", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     send_key(KeyCode::Char('a'), &event_sender); // Toggle all off
@@ -976,7 +975,7 @@ async fn test_results_calculation_with_files_changed_errors() -> anyhow::Result<
     send_chars("changed", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     overwrite_files!(
@@ -1068,7 +1067,7 @@ async fn test_results_calculation_with_files_deleted_errors() -> anyhow::Result<
     }
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     delete_files!(&temp_dir.path(), "src/lib.rs", "src/foo.rs");
@@ -1152,7 +1151,7 @@ async fn test_results_calculation_with_directory_deleted_errors() -> anyhow::Res
     send_chars("changed", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     delete_files!(&temp_dir.path(), "src/");
@@ -1254,26 +1253,11 @@ async fn test_prepopulated_fields() -> anyhow::Result<()> {
             value: "new_value",
             set_by_cli: true,
         },
-        fixed_strings: FieldValue {
-            value: true,
-            set_by_cli: false,
-        },
         match_whole_word: FieldValue {
             value: false,
             set_by_cli: true,
         },
-        match_case: FieldValue {
-            value: false,
-            set_by_cli: false,
-        },
-        include_files: FieldValue {
-            value: "",
-            set_by_cli: false,
-        },
-        exclude_files: FieldValue {
-            value: "",
-            set_by_cli: false,
-        },
+        ..SearchFieldValues::default()
     };
 
     let config = AppConfig {
@@ -1281,9 +1265,11 @@ async fn test_prepopulated_fields() -> anyhow::Result<()> {
         search_field_values,
         ..AppConfig::default()
     };
-    let (run_handle, event_sender, mut snapshot_rx) = build_test_runner_with_config(config)?;
+    let (run_handle, event_sender, mut snapshot_rx) =
+        build_test_runner_with_config_and_width(config, 48)?;
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Search text"), 100).await?;
+    // Search should happen automatically as the search field was prepopulated
+    wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     // Pre-populated fields should be skipped when tabbing
     send_key(KeyCode::Tab, &event_sender);
@@ -1293,7 +1279,7 @@ async fn test_prepopulated_fields() -> anyhow::Result<()> {
 
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     send_key(KeyCode::Enter, &event_sender);
@@ -1348,7 +1334,7 @@ async fn test_replacement_progress_display() -> anyhow::Result<()> {
     send_chars("TEST", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     send_key(KeyCode::Enter, &event_sender);
@@ -1418,7 +1404,7 @@ test_with_both_regex_modes!(
         };
         let (run_handle, event_sender, mut snapshot_rx) = build_test_runner_with_config(config)?;
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 100).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         let snapshot =
             wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 500).await?;
         assert!(Regex::new(r"file1\.txt").unwrap().is_match(&snapshot),);
@@ -1484,7 +1470,7 @@ test_with_both_regex_modes!(
         send_chars("REPLACEMENT", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         // Replacement should happen without confirmation
         wait_for_match(&mut snapshot_rx, Pattern::string("Success!"), 1000).await?;
 
@@ -1551,7 +1537,7 @@ test_with_both_regex_modes!(
         send_chars("123,", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -1624,7 +1610,7 @@ test_with_both_regex_modes!(
         send_key(KeyCode::Char(' '), &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -1698,7 +1684,7 @@ test_with_both_regex_modes!(
         send_key(KeyCode::Char(' '), &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -1780,7 +1766,7 @@ test_with_both_regex_modes!(
         send_key(KeyCode::Char(' '), &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -1851,7 +1837,7 @@ test_with_both_regex_modes!(
         send_chars("VERB", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -1921,7 +1907,7 @@ test_with_both_regex_modes!(
         send_key(KeyCode::Char(' '), &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -2058,7 +2044,7 @@ async fn test_advanced_regex_negative_lookahead() -> anyhow::Result<()> {
     send_chars("BAR", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     send_key(KeyCode::Enter, &event_sender);
@@ -2141,7 +2127,7 @@ test_with_both_regex_modes!(
         send_chars("dir2/*, dir3/**, */subdir3/*", &event_sender); // "Files to include" field
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -2236,7 +2222,7 @@ test_with_both_regex_modes!(
         send_chars("dir1", &event_sender); // "Files to exclude" field
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -2335,7 +2321,7 @@ test_with_both_regex_modes!(
         send_chars("**/file2.rs", &event_sender); // "Files to exclude" field
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -2448,7 +2434,7 @@ test_with_both_regex_modes!(
         send_chars("**/file2.rs", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -2531,7 +2517,7 @@ test_with_both_regex_modes!(test_ignores_gif_file, |advanced_regex: bool| async
     send_chars("", &event_sender);
     send_key(KeyCode::Enter, &event_sender);
 
-    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+    wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
     wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
     send_key(KeyCode::Enter, &event_sender);
@@ -2585,7 +2571,7 @@ test_with_both_regex_modes!(
         send_chars("REPLACED", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -2641,7 +2627,7 @@ test_with_both_regex_modes!(
         send_chars("REPLACED", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -2709,7 +2695,7 @@ test_with_both_regex_modes!(
         send_chars("REPLACED", &event_sender);
         send_key(KeyCode::Enter, &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::Enter, &event_sender);
@@ -2866,7 +2852,7 @@ test_with_both_regex_modes!(
         send_key(KeyCode::Tab, &event_sender);
         send_chars("bux", &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         send_key(KeyCode::BackTab, &event_sender);
@@ -2880,13 +2866,13 @@ test_with_both_regex_modes!(
         }
         send_chars("baz", &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         // Don't wait for "Search complete", start searching again immediately
 
         send_key(KeyCode::Backspace, &event_sender);
         send_chars("r", &event_sender);
 
-        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 500).await?;
+        wait_for_match(&mut snapshot_rx, Pattern::string("Still searching"), 1000).await?;
         wait_for_match(&mut snapshot_rx, Pattern::string("Search complete"), 1000).await?;
 
         let (mid_foo_count, mid_bar_count) = count_occurrences(temp_dir.path())?;
